%************************************************************************************************************************************************ 
%***************************************************** Model of rat atrial cardiomyocyte 2016 ***************************************************
                                       %Electrophysiological activity at 5 Hz steady-state stimulation
                %Published by Bbarov M, Matejovi P, Psek M, et al.: Naunyn Schmiedebergs Arch Pharmacol 2016;389:1049-1058  
%************************************************************************************************************************************************ 
                                                               %Compiled in MATLAB R2011b
%************************************************************************************************************************************************
                                                                  %ALL RIGHTS RESERVED
     %For research use only; commercial use prohibited. Distribution without permission of Michal Psek (mpasek@med.muni.cz) not permitted. 
%************************************************************************************************************************************************



%*****Global variables*****
global R T F Vel Rel Const_1
global Smtot Sms Smt Cms Cmt
global Nae Cae Ke BTRHmax BTRLmax ATPi ExBUFFERp Cap Achs
global gNa gNas PKsCa PKsCas gK1 gK1s gKb gKbs PKto PKtos PKss PKsss gKach gKachs gKsNa gKsNas gKatp gKatps gNab gNabs gCab gCabs 
global INaCamax INaCamaxs INaCamaxds INaKmax INaKmaxs IpCamax IpCamaxs  
global PCa PCas PCads PNal PNals PKl PKls PnsNa PnsNas PnsK PnsKs
global fCads fNaCads fVSRrel_c;
global t0 
global VCLAMP STIMULATION DIALYSIS



%Physical constants 
R=8310; T=(273+22); F=96500; 

%Total membrane area [cm^2]
Smtot=0.00005;

%Surface to volume ratio constant [cm^2/ml]
Const_1=8979;

%Concentrations [mM]
Nae=140; Cae=1.2; Ke=5.4; ATPi=6.8; Achs=0;
Cap=0.0000005; ExBUFFERp=0;
BTRHmax=0.14; BTRLmax=0.07; 

%Dialysis of cell by electrode [0 or 1]
DIALYSIS=0;

%Conductivities, permeabilities or maximum current densities 
gNa=13;          %[mS/cm^2]                 
PCa=0.00012012;  %[cm/s]                     
PNal=0;          %[cm/s]                    
PKl=0;           %[cm/s] 
PKto=0.00000023; %[cm/s]                 
PKss=0.000000342;%[cm/s]
gK1=0.223;       %[mS/cm^2]
gKach=0.10059;   %[mS/cm^2]                                                                                                                                                                                       
gNab=0.0008015;  %[mS/cm^2]                     
gCab=0.000648;   %[mS/cm^2]                 
gKb=0.00138;     %[mS/cm^2]   
INaCamax=0.00018;%[uA/cm^2]
INaKmax=1;       %[uA/cm^2] 
IpCamax=0.425;   %[uA/cm^2]                

PnsNa=0;         %[cm/s]                     
PnsK=0;          %[cm/s]  
PKsCa=0;         %[cm/s] 
gKsNa=0;         %[mS/cm^2]                
gKatp=0;         %[mS/cm^2]                

%Fraction of ICa at dyads [-]
fCads=0.6; 

%Fraction of INaCa at dyads [-]
fNaCads=0;

%Fraction of corbular SRrel [-]
fVSRrel_c=0.8;

%Serial resistance of electrode [Ohm]
Rel=2000000;

%Switching between current clamp and voltage clamp [0 and 1]
VCLAMP=0; 

%Maximal integration step     
Tinmax=0.1;

%Graphic output time
Graph_time=0;

%Assci file of final conditions [0 and 1]
Fincon=0;

%Assci file of results [0 and 1]
Results=0;



%Initial conditions for 5 Hz steady state (stabilised 1200 s)
ICon=[
 -8.1245902565031002e+001
  8.8989346747966064e-007
  6.3493126699480595e-001
  1.0000000000000000e+000
  3.3448505293765869e-002
  9.3521273186297305e-001
  2.9146664983959325e-001
  9.0784799000000005e-001
  7.3810678279870381e-002
  9.9995888833271296e-001
  4.8403358000000003e-005
  2.8691487999999999e-007
  1.8709789339433606e-006
  2.5871575070583804e-002
  7.7518483999999995e-004
  1.1038419000000001e-004
  6.9844693999999997e-006
  1.6570275999999999e-007
  1.1837564000000000e-010
  9.7990896000000005e-001
 -7.9780340448213479e+001
  2.0964643776813897e-006
  5.8647362871458641e-001
  3.0968868346521139e-001
  2.3340282625227238e-003
  9.8492197543379723e-001
  7.7604745281482912e-001
  3.4714088906825286e-003
  3.4714315766479664e-003
  9.9785393000000000e-001
  6.7585875965122749e-002
  9.9994617140962627e-001
  4.8359246369368097e-001
  3.6341766999999998e-020
  2.5078880465608665e-006
  6.3748071272019008e-002
  4.5873739999999998e-007
  3.8024055999999997e-011
  1.4007749000000000e-015
  1.9351217000000000e-020
  1.3823107000000000e-023
  9.8616565000000000e-001
  5.3178938649985730e+000
  1.4007771497887049e+002
  1.2159051948987012e+000
  1.3751298544734962e+002
  7.0457561877092987e+000
  2.4976628787557340e-005
  2.5361617596637673e-005
  9.8845686910452824e-001
  1.6372576308061090e-001
  6.4615006579187695e-001
  3.5587674051220003e-001
  6.6284582731784270e-001
  8.7965236516729439e-006
  4.4741203669437883e-006
  3.3714090203814234e-001
  1.0000000000000000e+000
  3.0894257091118875e-003
  0.0000000000000000e+000
  0.0000000000000000e+000
  0.0000000000000000e+000
  0.0000000000000000e+000
  1.7419097351265274e-004
  1.8069839271186257e+001
  1.2773732311979298e+002
 -3.4148136748522478e-125
 -7.4860700117842030e-125
  3.0459989516229630e-001
  4.9591097402460627e-001
  1.7848097063865165e-006
  1.8138849867745763e-006
  5.0408542728069472e-001
  1.8683375496353135e-004
  1.2773757994244397e+002
  1.8069625934736084e+001
  1.7031147740709652e-004
 -3.4147851669885657e-125
 -7.4860033146366045e-125
 -3.4147858076844078e-125
 -7.4860024991543670e-125
  1.2156659650496562e+000
  1.2154308827032163e+000
  1.2151998986575916e+000
  1.2149729640347160e+000
  1.2147500301909608e+000
  1.2145310486119349e+000
  1.2143159707878799e+000
  1.2141047480652920e+000
  5.3192579670462194e+000
  5.3205983089293332e+000
  5.3219151870173222e+000
  5.3232088961228845e+000
  5.3244797300573703e+000
  5.3257279822830110e+000
  5.3269539466830045e+000
  5.3281579184757071e+000
  ];


Vms0=ICon(1);  
ms0=ICon(2);
hs0=ICon(3);
ssts0=ICon(4); 
m_Ito_fast0=ICon(5); 
h_Ito_fast0=ICon(6); 
m_Ito_slow0=ICon(7); 
Cst0=ICon(8); 
CMDds0=ICon(9); 
VDI_s0=ICon(10); 
C30=ICon(11); 
C40=ICon(12); 
VDA_s0=ICon(13); 
CDI_ss0=ICon(14); 
Cca10=ICon(15); 
Cca20=ICon(16); 
Cca30=ICon(17); 
Cca40=ICon(18); 
Ccao0=ICon(19); 
ys0=ICon(20); 
Vmt0=ICon(21); 
mt0=ICon(22);
ht0=ICon(23);
sstt0=ICon(24); 
rt0=ICon(25);
st0=ICon(26);
h_Ito_slow0=ICon(27);
qs0=ICon(28);
qt0=ICon(29);
TCst0=ICon(30);
CMDss0=ICon(31);
VDI_t0=ICon(32);
TC30=ICon(33);
TC40=ICon(34);
VDA_t0=ICon(35);
CDI_ds0=ICon(36);
TCca10=ICon(37);
TCca20=ICon(38);
TCca30=ICon(39);
TCca40=ICon(40);
TCcao0=ICon(41);
yt0=ICon(42);
Kt_10=ICon(43);  
Nat0=ICon(44); 
Cat_10=ICon(45);
Kst0=ICon(46); 
Nast0=ICon(47); 
Cast0=ICon(48);
Cadt0=ICon(49); 
BTRH0=ICon(50);  
BTRL0=ICon(51);
CaSRup0=ICon(52);
CaSRrel_c0=ICon(53);    
F1t0=ICon(54); 
F2t0=ICon(55); 
F3t0=ICon(56);     
F4t0=ICon(57);  
rsts0=ICon(58); 
rstt0=ICon(59); 
ExBUFFERdt_free0=ICon(60);
ExBUFFERdt_Ca0=ICon(61);
ExBUFFERst_free0=ICon(62);
ExBUFFERst_Ca0=ICon(63);
Cac0=ICon(64);
Nac0=ICon(65);
Kc0=ICon(66);
ExBUFFERc_free0=ICon(67);
ExBUFFERc_Ca0=ICon(68);
CaSRrel_s0=ICon(69);    
F1s0=ICon(70); 
F2s0=ICon(71); 
F3s0=ICon(72);     
F4s0=ICon(73);  
Cads0=ICon(74); 
Kss0=ICon(75); 
Nass0=ICon(76); 
Cass0=ICon(77);
ExBUFFERds_free0=ICon(78);
ExBUFFERds_Ca0=ICon(79);
ExBUFFERss_free0=ICon(80);
ExBUFFERss_Ca0=ICon(81);
Cat_20=ICon(82);
Cat_30=ICon(83);
Cat_40=ICon(84);
Cat_50=ICon(85);
Cat_60=ICon(86);
Cat_70=ICon(87);
Cat_80=ICon(88);
Cat_90=ICon(89);
Kt_20=ICon(90);
Kt_30=ICon(91);
Kt_40=ICon(92);
Kt_50=ICon(93);
Kt_60=ICon(94);
Kt_70=ICon(95);
Kt_80=ICon(96);
Kt_90=ICon(97);

RMT=[];



%***********************  Solution 1  ************************
%Command voltage (only for VCLAMP=1)
Vel=-80;
%Change of extracellular concentrations
Ke=5.4; Cae=1.2; Nae=140;
%Stimulation option (only for VCLAMP=0)
STIMULATION=1;
%First interval of simulation
Tstart=0; Tend=0.2;
%Parameters of integration
InitialStep=0.0001; MaxStep=Tinmax; TOL=1e-6; Refinement=1; trace=0;
%Simulation
t0=Tstart; tend=Tend; graph 


%***********************  Solution 2  ************************
%Command voltage (only for VCLAMP=1)
Vel=0; 
%Change of extracellular concentrations
Ke=5.4; Cae=1.2; Nae=140;
%Stimulation option (only for VCLAMP=0)
STIMULATION=1;
%Interval of simulation
Tstart=0.2; Tend=0.4; 
%Stimulation period 
To=0.2;
for I=Tstart:To:Tend;
%Parameters of integration
InitialStep=0.0001; MaxStep=Tinmax; TOL=1e-6; Refinement=1; trace=0;
%Simulation
I
t0=I; tend=I+To; graph
end



RC=[
Vms(length(Vms));
ms(length(ms));
hs(length(hs));
ssts(length(ssts));
m_Ito_fast(length(m_Ito_fast));
h_Ito_fast(length(h_Ito_fast));
m_Ito_slow(length(m_Ito_slow));
Cst(length(Cst));
CMDds(length(CMDds));
VDI_s(length(VDI_s));
C3(length(C3));
C4(length(C4));
VDA_s(length(VDA_s));
CDI_ss(length(CDI_ss));
Cca1(length(Cca1));
Cca2(length(Cca2));
Cca3(length(Cca3));
Cca4(length(Cca4));
Ccao(length(Ccao));
ys(length(ys));
Vmt(length(Vmt));
mt(length(mt));
ht(length(ht));
sstt(length(sstt));
rt(length(rt));
st(length(st));
h_Ito_slow(length(h_Ito_slow));
qs(length(qs));
qt(length(qt));
TCst(length(TCst));
CMDss(length(CMDss));
VDI_t(length(VDI_t));
TC3(length(TC3));
TC4(length(TC4));
VDA_t(length(VDA_t));
CDI_ds(length(CDI_ds));
TCca1(length(TCca1));
TCca2(length(TCca2));
TCca3(length(TCca3));
TCca4(length(TCca4));
TCcao(length(TCcao));
yt(length(yt));
Kt_1(length(Kt_1));
Nat(length(Nat));
Cat_1(length(Cat_1));
Kst(length(Kst));
Nast(length(Nast));
Cast(length(Cast));
Cadt(length(Cadt));
BTRH(length(BTRH));
BTRL(length(BTRL));
CaSRup(length(CaSRup));
CaSRrel_c(length(CaSRrel_c));
F1t(length(F1t));
F2t(length(F2t));
F3t(length(F3t));
F4t(length(F4t));
rsts(length(rsts)); 
rstt(length(rstt)); 
ExBUFFERdt_free(length(ExBUFFERdt_free));
ExBUFFERdt_Ca(length(ExBUFFERdt_Ca));
ExBUFFERst_free(length(ExBUFFERst_free));
ExBUFFERst_Ca(length(ExBUFFERst_Ca));
Cac(length(Cac));
Nac(length(Nac));
Kc(length(Kc));
ExBUFFERc_free(length(ExBUFFERc_free));
ExBUFFERc_Ca(length(ExBUFFERc_Ca));
CaSRrel_s(length(CaSRrel_s));    
F1s(length(F1s));
F2s(length(F2s));
F3s(length(F3s));
F4s(length(F4s));
Cads(length(Cads));
Kss(length(Kss));
Nass(length(Nass));
Cass(length(Cass));
ExBUFFERds_free(length(ExBUFFERds_free));
ExBUFFERds_Ca(length(ExBUFFERds_Ca));
ExBUFFERss_free(length(ExBUFFERss_free));
ExBUFFERss_Ca(length(ExBUFFERss_Ca));
Cat_2(length(Cat_2));
Cat_3(length(Cat_3));
Cat_4(length(Cat_4));
Cat_5(length(Cat_5));
Cat_6(length(Cat_6));
Cat_7(length(Cat_7));
Cat_8(length(Cat_8));
Cat_9(length(Cat_9));
Kt_2(length(Kt_2));
Kt_3(length(Kt_3));
Kt_4(length(Kt_4));
Kt_5(length(Kt_5));
Kt_6(length(Kt_6));
Kt_7(length(Kt_7));
Kt_8(length(Kt_8));
Kt_9(length(Kt_9));
];

if Fincon==1
save fincon5Hz.asc RC -ascii -double;
end;

if Results==1
save results5Hz.asc RMT -ascii;
end;